@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/common/md5.h,v 1.8.2.2 2006/01/18 18:07:26 sev Exp $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/md5.h $
+ * $Id: md5.h 46108 2009-11-23 22:30:22Z fingolfin $
  */
 
 #ifndef COMMON_MD5_H
@@ -25,17 +29,15 @@
 
 namespace Common {
 
-typedef struct {
-	uint32 total[2];
-	uint32 state[4];
-	uint8 buffer[64];
-} md5_context;
+class ReadStream;
 
-void md5_starts(md5_context *ctx);
-void md5_update(md5_context *ctx, const uint8 *input, uint32 length);
-void md5_finish(md5_context *ctx, uint8 digest[16]);
+bool md5_file(ReadStream &stream, uint8 digest[16], uint32 length = 0);
 
-bool md5_file(const char *name, uint8 digest[16], const char *directory = NULL, uint32 length = 0);
+// The following method work similar to the above one, but
+// instead of computing the binary MD5 digest, it produces
+// a human readable lowercase hexstring representing the digest.
+// The md5str parameter must point to a buffer of 32+1 chars.
+bool md5_file_string(ReadStream &stream, char *md5str, uint32 length = 0);
 
 } // End of namespace Common
 
