@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/common/system.h,v 1.104.2.2 2006/01/18 18:07:26 sev Exp $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/system.h $
+ * $Id: system.h 47541 2010-01-25 01:39:44Z lordhoto $
  *
  */
 
@@ -24,18 +27,50 @@
 #define COMMON_SYSTEM_H
 
 #include "common/scummsys.h"
-#include "common/mutex.h"
+#include "common/noncopyable.h"
 #include "common/rect.h"
-#include "common/singleton.h"
+
+#include "graphics/pixelformat.h"
+
+namespace Audio {
+	class Mixer;
+}
 
 namespace Graphics {
 	struct Surface;
 }
 
 namespace Common {
+	struct Event;
+	class EventManager;
 	class SaveFileManager;
+	class SearchSet;
+	class TimerManager;
+	class SeekableReadStream;
+	class WriteStream;
+	class HardwareKeySet;
 }
 
+class FilesystemFactory;
+
+/**
+ * A structure describing time and date. This is a clone of struct tm
+ * from time.h. We roll our own since not all systems provide time.h.
+ * We also do not imitate all files of struct tm, only those we
+ * actually need.
+ *
+ * @note For now, the members are named exactly as in struct tm to ease
+ * the transition.
+ */
+struct TimeDate {
+	int tm_sec;     ///< seconds (0 - 60)
+	int tm_min;     ///< minutes (0 - 59)
+	int tm_hour;    ///< hours (0 - 23)
+	int tm_mday;    ///< day of month (1 - 31)
+	int tm_mon;     ///< month of year (0 - 11)
+	int tm_year;    ///< year - 1900
+};
+
 /**
  * Interface for ScummVM backends. If you want to port ScummVM to a system
  * which is not currently covered by any of our backends, this is the place
@@ -46,19 +81,35 @@ namespace Common {
  * methods to create timers, to handle user input events,
  * control audio CD playback, and sound output.
  */
-class OSystem : public Common::Singleton<OSystem> {
+class OSystem : Common::NonCopyable {
 protected:
-	static OSystem *makeInstance();
-	friend class Common::Singleton<SingletonBaseType>;
+	OSystem();
+	virtual ~OSystem();
 
 public:
 
 	/**
 	 * The following method is called once, from main.cpp, after all
 	 * config data (including command line params etc.) are fully loaded.
+	 *
+	 * @note Subclasses should always invoke the implementation of their
+	 *       parent class. They should do so near the end of their own
+	 *       implementation.
 	 */
 	virtual void initBackend() { }
 
+	/**
+	 * Allows the backend to perform engine specific init.
+	 * Called just before the engine is run.
+	 */
+	virtual void engineInit() { }
+
+	/**
+	 * Allows the backend to perform engine specific de-init.
+	 * Called after the engine finishes.
+	 */
+	virtual void engineDone() { }
+
 	/** @name Feature flags */
 	//@{
 
@@ -103,7 +154,7 @@ public:
 		 * the frontend code is very inefficient in doing screen updates. So
 		 * the frontend might do a lot of fullscreen blits even though only a
 		 * tiny portion of the actual screen data changed. In that case, it
-		 * might pay off for the backend to compute which parts actual changed,
+		 * might pay off for the backend to compute which parts actually changed,
 		 * and then only mark those as dirty.
 		 * Implementing this is purely optional, and no harm should arise
 		 * when not doing so (except for decreased speed in said frontends).
@@ -111,17 +162,43 @@ public:
 		kFeatureAutoComputeDirtyRects,
 
 		/**
-		 * This flags determines either cursor can have its own palette or not
+		 * This flag determines whether or not the cursor can have its own palette.
 		 * It is currently used only by some Macintosh versions of Humongous
-		 * Entertainment games. If backend doesn't implement this feature then
-		 * engine switches to b/w version of cursors.
+		 * Entertainment games. If the backend doesn't implement this feature then
+		 * the engine switches to b/w versions of cursors.
+		 * The GUI also relies on this feature for mouse cursors.
+		 *
+		 * To enable the cursor palette call "disableCursorPalette" with false.
+		 * @see disableCursorPalette
 		 */
 		kFeatureCursorHasPalette,
 
 		/**
 		 * Set to true if the overlay pixel format has an alpha channel.
+		 * This should only be set if it offers at least 3-4 bits of accuracy,
+		 * as opposed to a single alpha bit.
 		 */
-		kFeatureOverlaySupportsAlpha
+		kFeatureOverlaySupportsAlpha,
+
+		/**
+		 * Set to true to iconify the window.
+		 */
+		kFeatureIconifyWindow,
+
+		/**
+		 * This feature, set to true, is a hint toward the backend to disable all
+		 * key filtering/mapping, in cases where it would be beneficial to do so.
+		 * As an example case, this is used in the agi engine's predictive dialog.
+		 * When the dialog is displayed this feature is set so that backends with
+		 * phone-like keypad temporarily unmap all user actions which leads to
+		 * comfortable word entry. Conversely, when the dialog exits the feature
+		 * is set to false.
+		 * TODO: Fingolfin suggests that the way the feature is used can be
+		 * generalized in this sense: Have a keyboard mapping feature, which the
+		 * engine queries for to assign keys to actions ("Here's my default key
+		 * map for these actions, what do you want them set to?").
+		 */
+		kFeatureDisableKeyFiltering
 	};
 
 	/**
@@ -166,9 +243,9 @@ public:
 	 * game layer. Let W and H denote the width and height of the
 	 * game graphics.
 	 *
-	 * Before the user sees these graphics, they may undergo certain
-	 * transformations; for example, the may be scaled to better fit
-	 * on the visible screen; or aspect ratio correction may be
+	 * Before the user sees these graphics, the backend may apply some
+	 * transformations to it; for example, the may be scaled to better
+	 * fit on the visible screen; or aspect ratio correction may be
 	 * performed (see kFeatureAspectRatioCorrection). As a result of
 	 * this, a pixel of the game graphics may occupy a region bigger
 	 * than a single pixel on the screen. We define p_w and p_h to be
@@ -181,7 +258,7 @@ public:
 	 * by S.
 	 *
 	 * Putting this together, a pixel (x,y) of the game graphics is
-	 * transformed to a rectangle of height p_h and widht p_w
+	 * transformed to a rectangle of height p_h and width p_w
 	 * appearing at position (p_w * x, p_hw * (y + S)) on the real
 	 * screen (in addition, a backend may choose to offset
 	 * everything, e.g. to center the graphics on the screen).
@@ -197,6 +274,9 @@ public:
 	 * graphics have a resolution of 320x200; then the overlay shall
 	 * have a resolution of 640x400, but it still has the same
 	 * physical size as the game graphics.
+	 * The overlay usually uses 16bpp, but on some ports, only 8bpp
+	 * are availble, so that is supported, too, via a compile time
+	 * switch (see also the OverlayColor typedef in scummsys.h).
 	 *
 	 *
 	 * Finally, there is the mouse layer. This layer doesn't have to
@@ -285,8 +365,50 @@ public:
 	 */
 	virtual int getGraphicsMode() const = 0;
 
+#ifdef USE_RGB_COLOR
 	/**
-	 * Set the size of the virtual screen. Typical sizes include:
+	 * Determine the pixel format currently in use for screen rendering.
+	 * @return the active screen pixel format.
+	 * @see Graphics::PixelFormat
+	 */
+	virtual Graphics::PixelFormat getScreenFormat() const = 0;
+
+	/**
+	 * Returns a list of all pixel formats supported by the backend.
+	 * The first item in the list must be directly supported by hardware,
+	 * and provide the largest color space of those formats with direct
+	 * hardware support. It is also strongly recommended that remaining
+	 * formats should be placed in order of descending preference for the
+	 * backend to use.
+	 *
+	 * EG: a backend that supports 32-bit ABGR and 16-bit 555 BGR in hardware
+	 * and provides conversion from equivalent RGB(A) modes should order its list
+	 *    1) Graphics::PixelFormat(4, 0, 0, 0, 0, 0, 8, 16, 24)
+	 *    2) Graphics::PixelFormat(2, 3, 3, 3, 8, 0, 5, 10, 0)
+	 *    3) Graphics::PixelFormat(4, 0, 0, 0, 0, 24, 16, 8, 0)
+	 *    4) Graphics::PixelFormat(2, 3, 3, 3, 8, 10, 5, 0, 0)
+	 *    5) Graphics::PixelFormat::createFormatCLUT8()
+	 *
+	 * @see Graphics::PixelFormat
+	 *
+	 * @note Backends supporting RGB color should accept game data in RGB color
+	 *       order, even if hardware uses BGR or some other color order.
+	 */
+	virtual Common::List<Graphics::PixelFormat> getSupportedFormats() = 0;
+#else
+	inline Graphics::PixelFormat getScreenFormat() const {
+		return Graphics::PixelFormat::createFormatCLUT8();
+	};
+
+	inline Common::List<Graphics::PixelFormat> getSupportedFormats() const {
+		Common::List<Graphics::PixelFormat> list;
+		list.push_back(Graphics::PixelFormat::createFormatCLUT8());
+		return list;
+	};
+#endif
+
+	/**
+	 * Set the size and color format of the virtual screen. Typical sizes include:
 	 *  - 320x200 (e.g. for most SCUMM games, and Simon)
 	 *  - 320x240 (e.g. for FM-TOWN SCUMM games)
 	 *  - 640x480 (e.g. for Curse of Monkey Island)
@@ -297,13 +419,36 @@ public:
 	 * GraphicsMode); stretch the data to perform aspect ratio correction;
 	 * or shrink it to fit on small screens (in cell phones).
 	 *
+	 * Typical formats include:
+	 *  CLUT8 (e.g. 256 color, for most games)
+	 *  RGB555 (e.g. 16-bit color, for later SCUMM HE games)
+	 *  RGB565 (e.g. 16-bit color, for Urban Runner)
+	 *
+	 * This is the pixel format for which the client code generates data;
+	 * this is not necessarily equal to the hardware pixel format. For example,
+	 * a backend may perform color lookup of 8-bit graphics before pushing
+	 * a screen to hardware, or correct the ARGB color order.
+	 *
 	 * @param width		the new virtual screen width
 	 * @param height	the new virtual screen height
-	 * @param overlayScale	optional: the scale to be used for the overlay, if different
-	 *						from the default scale (default is -1, which means the
-	 *						overlay uses the same scale)
+	 * @param format	the new virtual screen pixel format
 	 */
-	virtual void initSize(uint width, uint height, int overlayScale = -1) = 0;
+	virtual void initSize(uint width, uint height, const Graphics::PixelFormat *format = NULL) = 0;
+
+	/**
+	 * Return an int value which is changed whenever any screen
+	 * parameters (like the resolution) change. That is, whenever a
+	 * EVENT_SCREEN_CHANGED would be sent. You can track this value
+	 * in your code to detect screen changes in case you do not have
+	 * full control over the event loop(s) being used (like the GUI
+	 * code).
+	 *
+	 * @return an integer which can be used to track screen changes
+	 *
+	 * @note Backends which generate EVENT_SCREEN_CHANGED events MUST
+	 *       overload this method appropriately.
+	 */
+	virtual int getScreenChangeID() const { return 0; }
 
 	/**
 	 * Begin a new GFX transaction, which is a sequence of GFX mode changes.
@@ -321,14 +466,33 @@ public:
 	 *       of the relevant methods simply do nothing.
 	 * @see endGFXTransaction
 	 */
-	virtual void beginGFXTransaction() {};
+	virtual void beginGFXTransaction() {}
 
+	/**
+	 * This type is able to save the different errors which can happen while
+	 * changing GFX config values inside GFX transactions.
+	 *
+	 * endGFXTransaction returns a ORed combination of the '*Failed' values
+	 * if any problem occures, on success 0.
+	 *
+	 * @see endGFXTransaction
+	 */
+	enum TransactionError {
+		kTransactionSuccess = 0,					/**< Everything fine (use EQUAL check for this one!) */
+		kTransactionAspectRatioFailed = (1 << 0),	/**< Failed switching aspect ratio correction mode */
+		kTransactionFullscreenFailed = (1 << 1),	/**< Failed switching fullscreen mode */
+		kTransactionModeSwitchFailed = (1 << 2),	/**< Failed switching the GFX graphics mode (setGraphicsMode) */
+		kTransactionSizeChangeFailed = (1 << 3),	/**< Failed switching the screen dimensions (initSize) */
+		kTransactionFormatNotSupported = (1 << 4)	/**< Failed setting the color format */
+	};
 
 	/**
 	 * End (and thereby commit) the current GFX transaction.
 	 * @see beginGFXTransaction
+	 * @see kTransactionError
+	 * @return returns a ORed combination of TransactionError values or 0 on success
 	 */
-	virtual void endGFXTransaction() {};
+	virtual TransactionError endGFXTransaction() { return kTransactionSuccess; }
 
 
 	/**
@@ -352,12 +516,12 @@ public:
 	 *
 	 * The palette data is specified in interleaved RGBA format. That is, the
 	 * first byte of the memory block 'colors' points at is the red component
-	 * of the first new color; the second byte the blue component of the first
-	 * new color; the third byte the green component, the last byte to the alpha
+	 * of the first new color; the second byte the green component of the first
+	 * new color; the third byte the blue component, the last byte to the alpha
 	 * (transparency) value. Then the second color starts, and so on. So memory
 	 * looks like this: R1-G1-B1-A1-R2-G2-B2-A2-R3-...
 	 *
-	 * @param colors	the new palette data, in interleaved RGB format
+	 * @param colors	the new palette data, in interleaved RGBA format
 	 * @param start		the first palette entry to be updated
 	 * @param num		the number of palette entries to be updated
 	 *
@@ -372,7 +536,8 @@ public:
 	 * Grabs a specified part of the currently active palette.
 	 * The format is the same as for setPalette.
 	 *
-	 * @param colors	the palette data, in interleaved RGB format
+	 * @see setPalette
+	 * @param colors	the palette data, in interleaved RGBA format
 	 * @param start		the first platte entry to be read
 	 * @param num		the number of palette entries to be read
 	 */
@@ -384,43 +549,101 @@ public:
 	 * Client code has to to call updateScreen to ensure any changes are
 	 * visible to the user. This can be used to optimize drawing and reduce
 	 * flicker.
+	 * The graphics data uses 8 bits per pixel, using the palette specified
+	 * via setPalette.
+	 *
+	 * @param buf		the buffer containing the graphics data source
+	 * @param pitch		the pitch of the buffer (number of bytes in a scanline)
+	 * @param x			the x coordinate of the destination rectangle
+	 * @param y			the y coordinate of the destination rectangle
+	 * @param w			the width of the destination rectangle
+	 * @param h			the height of the destination rectangle
+	 *
+	 * @note The specified destination rectangle must be completly contained
+	 *       in the visible screen space, and must be non-empty. If not, a
+	 *       backend may or may not perform clipping, trigger an assert or
+	 *       silently corrupt memory.
+	 *
 	 * @see updateScreen
 	 */
 	virtual void copyRectToScreen(const byte *buf, int pitch, int x, int y, int w, int h) = 0;
 
 	/**
-	 * Copies the current screen contents to a new surface, with the original
-	 * bit depth. This will allocate memory for the pixel data.
-	 * WARNING: surf->free() must be called by the user to avoid leaking.
+	 * Lock the active screen framebuffer and return a Graphics::Surface
+	 * representing it. The caller can then perform arbitrary graphics
+	 * transformations on the framebuffer (blitting, scrolling, etc.).
+	 * Must be followed by matching call to unlockScreen(). Calling code
+	 * should make sure to only lock the framebuffer for the briefest
+	 * periods of time possible, as the whole system is potentially stalled
+	 * while the lock is active.
+	 * Returns 0 if an error occurred. Otherwise an 8bit surface is returned.
 	 *
-	 * @param surf	the surfce to store the data in it
-	 * @return true if all went well, false if an error occured
+	 * The returned surface must *not* be deleted by the client code.
 	 */
-	virtual bool grabRawScreen(Graphics::Surface *surf) { return false; }
+	virtual Graphics::Surface *lockScreen() = 0;
 
 	/**
-	 * Clear the screen to black.
+	 * Unlock the screen framebuffer, and mark it as dirty (i.e. during the
+	 * next updateScreen() call, the whole screen will be updated.
 	 */
-	virtual void clearScreen() {}
+	virtual void unlockScreen() = 0;
 
-	/** Update the dirty areas of the screen. */
+	/**
+	 * Fills the screen with a given color value.
+	 *
+	 * @note We are using uint32 here even though currently
+	 * we only support 8bpp indexed mode. Thus the value should
+	 * be always inside [0, 255] for now.
+	 */
+	virtual void fillScreen(uint32 col) = 0;
+
+	/**
+	 * Flush the whole screen, that is render the current content of the screen
+	 * framebuffer to the display.
+	 *
+	 * This method could be called very often by engines. Backends are hence
+	 * supposed to only perform any redrawing if it is necessary, and otherwise
+	 * return immediately. For a discussion of the subject, see
+	 * <http://www.nabble.com/ATTN-porters%3A-updateScreen%28%29-OSystem-method-tt3960261.html#a3960261>
+	 */
 	virtual void updateScreen() = 0;
 
 	/**
-	 * Set current shake position, a feature needed for some SCUMM screen effects.
-	 * The effect causes the displayed graphics to be shifted upwards by the specified
-	 * (always positive) offset. The area at the bottom of the screen which is moved
-	 * into view by this is filled by black. This does not cause any graphic data to
-	 * be lost - that is, to restore the original view, the game engine only has to
-	 * call this method again with a 0 offset. No calls to copyRectToScreen are necessary.
+	 * Set current shake position, a feature needed for some SCUMM screen
+	 * effects. The effect causes the displayed graphics to be shifted upwards
+	 * by the specified (always positive) offset. The area at the bottom of the
+	 * screen which is moved into view by this is filled with black. This does
+	 * not cause any graphic data to be lost - that is, to restore the original
+	 * view, the game engine only has to call this method again with offset
+	 * equal to zero. No calls to copyRectToScreen are necessary.
 	 * @param shakeOffset	the shake offset
 	 *
-	 * @todo This is a rather special screen effect, only used by the SCUMM
-	 *       frontend - we should consider removing it from the backend API
-	 *       and instead implement the functionality in the frontend.
+	 * @note This is currently used in the SCUMM, QUEEN and KYRA engines.
 	 */
 	virtual void setShakePos(int shakeOffset) = 0;
 
+	/**
+	 * Sets the area of the screen that has the focus.  For example, when a character
+	 * is speaking, they will have the focus.  Allows for pan-and-scan style views
+	 * where the backend could follow the speaking character or area of interest on
+	 * the screen.
+	 *
+	 * The backend is responsible for clipping the rectangle and deciding how best to
+	 * zoom the screen to show any shape and size rectangle the engine provides.
+	 *
+	 * @param rect A rectangle on the screen to be focused on
+	 * @see clearFocusRectangle
+	 */
+	virtual void setFocusRectangle(const Common::Rect& rect) {}
+
+	/**
+	 * Clears the focus set by a call to setFocusRectangle().  This allows the engine
+	 * to clear the focus during times when no particular area of the screen has the
+	 * focus.
+	 * @see setFocusRectangle
+	 */
+	virtual void clearFocusRectangle() {}
+
 	//@}
 
 
@@ -439,8 +662,10 @@ public:
 	 * 8bpp), this needs some trickery.
 	 *
 	 * Essentially, we fake (alpha) blending on these systems by copying the
-	 * game graphics into the overlay buffer, then manually compose whatever
-	 * graphics we want to show in the overlay.
+	 * current game graphics into the overlay buffer when activating the overlay,
+	 * then manually compose whatever graphics we want to show in the overlay.
+	 * This works because we assume the game to be "paused" whenever an overlay
+	 * is active.
 	 */
 	//@{
 
@@ -450,6 +675,12 @@ public:
 	/** Deactivate the overlay mode. */
 	virtual void hideOverlay() = 0;
 
+	/**
+	 * Returns the pixel format description of the overlay.
+	 * @see Graphics::PixelFormat
+	 */
+	virtual Graphics::PixelFormat getOverlayFormat() const = 0;
+
 	/**
 	 * Reset the overlay.
 	 *
@@ -472,6 +703,20 @@ public:
 	/**
 	 * Blit a graphics buffer to the overlay.
 	 * In a sense, this is the reverse of grabOverlay.
+	 *
+	 * @note The pitch parameter actually contains the 'pixel pitch', i.e.,
+	 * the number of pixels per scanline, and not as usual the number of bytes
+	 * per scanline.
+	 *
+	 * @todo Change 'pitch' to be byte and not pixel based
+	 *
+	 * @param buf		the buffer containing the graphics data source
+	 * @param pitch		the pixel pitch of the buffer (number of pixels in a scanline)
+	 * @param x			the x coordinate of the destination rectangle
+	 * @param y			the y coordinate of the destination rectangle
+	 * @param w			the width of the destination rectangle
+	 * @param h			the height of the destination rectangle
+	 *
 	 * @see copyRectToScreen
 	 * @see grabOverlay
 	 */
@@ -481,100 +726,62 @@ public:
 	 * Return the height of the overlay.
 	 * @see getHeight
 	 */
-	virtual int16 getOverlayHeight()  { return getHeight(); }
+	virtual int16 getOverlayHeight() = 0;
 
 	/**
 	 * Return the width of the overlay.
 	 * @see getWidth
 	 */
-	virtual int16 getOverlayWidth()   { return getWidth(); }
-
-	virtual int screenToOverlayX(int x) { return x; }
-	virtual int screenToOverlayY(int y) { return y; }
-	virtual int overlayToScreenX(int x) { return x; }
-	virtual int overlayToScreenY(int y) { return y; }
-
-	/**
-	* Convert the given RGB triplet into an OverlayColor. A OverlayColor can
-	 * be 8bit, 16bit or 32bit, depending on the target system. The default
-	 * implementation generates a 16 bit color value, in the 565 format
-	 * (that is, 5 bits red, 6 bits green, 5 bits blue).
-	 * @see colorToRGB
-	 * @see ARGBToColor
-	 */
-	virtual OverlayColor RGBToColor(uint8 r, uint8 g, uint8 b) {
-		return ((((r >> 3) & 0x1F) << 11) | (((g >> 2) & 0x3F) << 5) | ((b >> 3) & 0x1F));
-	}
-
-	/**
-	 * Convert the given OverlayColor into a RGB triplet. An OverlayColor can
-	 * be 8bit, 16bit or 32bit, depending on the target system. The default
-	 * implementation takes a 16 bit color value and assumes it to be in 565 format
-	 * (that is, 5 bits red, 6 bits green, 5 bits blue).
-	 * @see RGBToColor
-	 * @see colorToARGB
-	 */
-	virtual void colorToRGB(OverlayColor color, uint8 &r, uint8 &g, uint8 &b) {
-		r = (((color >> 11) & 0x1F) << 3);
-		g = (((color >> 5) & 0x3F) << 2);
-		b = ((color&0x1F) << 3);
-	}
-
-	/**
-	* Convert the given ARGB quadruplet into an OverlayColor. A OverlayColor can
-	 * be 8bit, 16bit or 32bit, depending on the target system. The default
-	 * implementation generates a 16 bit color value, in the 565 format
-	 * (that is, 5 bits red, 6 bits green, 5 bits blue).
-	 * @see colorToRGB
-	 * @see RGBToColor
-	 */
-	virtual OverlayColor ARGBToColor(uint8 a, uint8 r, uint8 g, uint8 b) {
-		return RGBToColor(r, g, b);
-	}
-
-	/**
-	 * Convert the given OverlayColor into an ARGB quadruplet. An OverlayColor can
-	 * be 8bit, 16bit or 32bit, depending on the target system. The default
-	 * implementation takes a 16 bit color value and assumes it to be in 565 format
-	 * (that is, 5 bits red, 6 bits green, 5 bits blue).
-	 * @see ARGBToColor
-	 * @see colorToRGB
-	 */
-	virtual void colorToARGB(OverlayColor color, uint8 &a, uint8 &r, uint8 &g, uint8 &b) {
-		colorToRGB(color, r, g, b);
-		a = 255;
-	}
+	virtual int16 getOverlayWidth() = 0;
 
 	//@}
 
 
 
-	/** @name Mouse */
+	/** @name Mouse
+	 * This is the lower level implementation as provided by the
+	 * backends. The engines should use the Graphics::CursorManager
+	 * class instead of using it directly.
+	 */
 	//@{
 
-	/** Show or hide the mouse cursor. */
+	/**
+	 * Show or hide the mouse cursor.
+	 *
+	 * Currently the backend is not required to immediately draw the
+	 * mouse cursor on showMouse(true).
+	 *
+	 * TODO: We might want to reconsider this fact,
+	 * check Graphics::CursorManager::showMouse for some details about
+	 * this.
+	 *
+	 * @see Graphics::CursorManager::showMouse
+	 */
 	virtual bool showMouse(bool visible) = 0;
 
 	/**
 	 * Move ("warp") the mouse cursor to the specified position in virtual
 	 * screen coordinates.
 	 * @param x		the new x position of the mouse
-	 * @param y		the new x position of the mouse
+	 * @param y		the new y position of the mouse
 	 */
 	virtual void warpMouse(int x, int y) = 0;
 
 	/**
 	 * Set the bitmap used for drawing the cursor.
 	 *
-	 * @param buf				the pixmap data to be used (8bit/pixel)
+	 * @param buf				the pixmap data to be used
 	 * @param w					width of the mouse cursor
 	 * @param h					height of the mouse cursor
 	 * @param hotspotX			horizontal offset from the left side to the hotspot
 	 * @param hotspotY			vertical offset from the top side to the hotspot
-	 * @param keycolor			transparency color index
+	 * @param keycolor			transparency color value. This should not exceed the maximum color value of the specified format.
+	 *                          In case it does the behavior is undefined. The backend might just error out or simply ignore the
+	 *                          value. (The SDL backend will just assert to prevent abuse of this).
 	 * @param cursorTargetScale	scale factor which cursor is designed for
+	 * @param format			pointer to the pixel format which cursor graphic uses (0 means CLUT8)
 	 */
-	virtual void setMouseCursor(const byte *buf, uint w, uint h, int hotspotX, int hotspotY, byte keycolor = 255, int cursorTargetScale = 1) = 0;
+	virtual void setMouseCursor(const byte *buf, uint w, uint h, int hotspotX, int hotspotY, uint32 keycolor, int cursorTargetScale = 1, const Graphics::PixelFormat *format = NULL) = 0;
 
 	/**
 	 * Replace the specified range of cursor the palette with new colors.
@@ -586,7 +793,7 @@ public:
 	 * @see setPalette
 	 * @see kFeatureCursorHasPalette
 	 */
-	virtual void setCursorPalette(const byte *colors, uint start, uint num) {};
+	virtual void setCursorPalette(const byte *colors, uint start, uint num) {}
 
 	/**
 	 * Disable or enable cursor palette.
@@ -598,7 +805,7 @@ public:
 	 * @see setPalette
 	 * @see kFeatureCursorHasPalette
 	 */
-	virtual void disableCursorPalette(bool disable) {};
+	virtual void disableCursorPalette(bool disable) {}
 
 	//@}
 
@@ -607,122 +814,6 @@ public:
 	/** @name Events and Time */
 	//@{
 
-	typedef int (*TimerProc)(int interval);
-
-	/**
-	 * The types of events backends may generate.
-	 * @see Event
-	 *
-	 * @todo Merge EVENT_LBUTTONDOWN, EVENT_RBUTTONDOWN and EVENT_WHEELDOWN;
-	 *       likewiese EVENT_LBUTTONUP, EVENT_RBUTTONUP, EVENT_WHEELUP.
-	 *       To do that, we just have to add a field to the Event which
-	 *       indicates which button was pressed.
-	 */
-	enum EventType {
-		/** A key was pressed, details in Event::kbd. */
-		EVENT_KEYDOWN = 1,
-		/** A key was released, details in Event::kbd. */
-		EVENT_KEYUP = 2,
-		/** The mouse moved, details in Event::mouse. */
-		EVENT_MOUSEMOVE = 3,
-		EVENT_LBUTTONDOWN = 4,
-		EVENT_LBUTTONUP = 5,
-		EVENT_RBUTTONDOWN = 6,
-		EVENT_RBUTTONUP = 7,
-		EVENT_WHEELUP = 8,
-		EVENT_WHEELDOWN = 9,
-
-		EVENT_QUIT = 10,
-		EVENT_SCREEN_CHANGED = 11
-	};
-
-	/**
-	 * Keyboard modifier flags, used for Event::kbd::flags.
-	 */
-	enum {
-		KBD_CTRL  = 1 << 0,
-		KBD_ALT   = 1 << 1,
-		KBD_SHIFT = 1 << 2
-	};
-
-	/**
-	 * Data structure for an event. A pointer to an instance of Event
-	 * can be passed to pollEvent.
-	 * @todo Rework/document this structure. It should be made 100% clear which
-	 *       field is valid for which event type.
-	 *       Implementation wise, we might want to use the classic
-	 *       union-of-structs trick. It goes roughly like this:
-	 *       struct BasicEvent {
-	 *       	EventType type;
-	 *       };
-	 *       struct MouseMovedEvent : BasicEvent {
-	 *       	Common::Point pos;
-	 *       };
-	 *       struct MouseButtonEvent : MouseMovedEvent {
-	 *       	int button;
-	 *       };
-	 *       struct KeyEvent : BasicEvent {
-	 *       	...
-	 *       };
-	 *       ...
-	 *       union Event {
-	 *          EventType type;
-	 *       	MouseMovedEvent mouse;
-	 *       	MouseButtonEvent button;
-	 *       	KeyEvent key;
-	 *       	...
-	 *       };
-	 */
-	struct Event {
-		/** The type of the event. */
-		EventType type;
-		/**
-		  * Keyboard data; only valid for keyboard events (EVENT_KEYDOWN and
-		  * EVENT_KEYUP). For all other event types, content is undefined.
-		  */
-		struct {
-			/**
-			 * Abstract key code (will be the same for any given key regardless
-			 * of modifiers being held at the same time.
-			 * For example, this is the same for both 'A' and Shift-'A'.
-			 * @todo Document which values are to be used for non-ASCII keys
-			 * like F1-F10. For now, let's just say that our primary backend
-			 * is the SDL one, and it uses the values SDL uses... so until
-			 * we fix this, your best bet is to get a copy of SDL_keysym.h
-			 * and look at that, if you want to find out a key code.
-			 */
-			int keycode;
-			/**
-			 * ASCII-value of the pressed key (if any).
-			 * This depends on modifiers, i.e. pressing the 'A' key results in
-			 * different values here depending on the status of shift, alt and
-			 * caps lock.
-			 * For the function keys F1-F9, values of 315-323 are used.
-			 */
-			uint16 ascii;
-			/**
-			 * Status of the modifier keys. Bits are set in this for each
-			 * pressed modifier
-			 * @see KBD_CTRL, KBD_ALT, KBD_SHIFT
-			 */
-			byte flags;
-		} kbd;
-		/**
-		 * The mouse coordinates, in virtual screen coordinates. Only valid
-		 * for mouse events.
-		 * Virtual screen coordinates means: the coordinate system of the
-		 * screen area as defined by the most recent call to initSize().
-		 */
-		Common::Point mouse;
-	};
-
-	/**
-	 * Get the next event in the event queue.
-	 * @param event	point to an Event struct, which will be filled with the event data.
-	 * @return true if an event was retrieved.
-	 */
-	virtual bool pollEvent(Event &event) = 0;
-
 	/** Get the number of milliseconds since the program was started. */
 	virtual uint32 getMillis() = 0;
 
@@ -730,24 +821,32 @@ public:
 	virtual void delayMillis(uint msecs) = 0;
 
 	/**
-	 * Set the timer callback, a function which is periodically invoked by the
-	 * backend. This can for example be done via a background thread.
-	 * There is at most one active timer; if this method is called while there
-	 * is already an active timer, then the new timer callback should replace
-	 * the previous one. In particular, passing a callback pointer value of 0
-	 * is legal and can be used to clear the current timer callback.
-	 * @see Common::Timer
-	 * @note The implementation of this method must be 'atomic' in the sense
-	 *       that when the method returns, the previously set callback must
-	 *       not be in use anymore (in particular, if timers are implemented
-	 *       via threads, then it must be ensured that the timer thread is
-	 *       not using the old callback function anymore).
-	 *
-	 * @param callback	pointer to the callback. May be 0 to reset the timer
-	 * @param interval	the interval (in milliseconds) between invocations
-	 *                  of the callback
+	 * Get the current time and date, in the local timezone.
+	 * Corresponds on many systems to the combination of time()
+	 * and localtime().
 	 */
-	virtual void setTimerCallback(TimerProc callback, int interval) = 0;
+	virtual void getTimeAndDate(TimeDate &t) const = 0;
+
+	/**
+	 * Return the timer manager singleton. For more information, refer
+	 * to the TimerManager documentation.
+	 */
+	virtual Common::TimerManager *getTimerManager() = 0;
+
+	/**
+	 * Return the event manager singleton. For more information, refer
+	 * to the EventManager documentation.
+	 */
+	virtual Common::EventManager *getEventManager() = 0;
+
+	/**
+	 * Register hardware keys with keymapper
+	 *
+	 * @return HardwareKeySet with all keys and recommended mappings
+	 *
+	 * See keymapper documentation for further reference.
+	 */
+	virtual Common::HardwareKeySet *getHardwareKeySet() { return 0; }
 
 	//@}
 
@@ -762,13 +861,15 @@ public:
 	 * But since those may be implemented using threads (and in fact, that's
 	 * how our primary backend, the SDL one, does it on many systems), we
 	 * still have to do mutex syncing in our timer callbacks.
+	 * In addition, the sound mixer uses a mutex in case the backend runs it
+	 * from a dedicated thread (as e.g. the SDL backend does).
 	 *
 	 * Hence backends which do not use threads to implement the timers simply
 	 * can use dummy implementations for these methods.
 	 */
 	//@{
 
-	typedef Common::MutexRef	MutexRef;
+	typedef struct OpaqueMutex *MutexRef;
 
 	/**
 	 * Create a new mutex.
@@ -778,6 +879,12 @@ public:
 
 	/**
 	 * Lock the given mutex.
+	 *
+	 * @note ScummVM code assumes that the mutex implementation supports
+	 * recursive locking. That is, a thread may lock a mutex twice w/o
+	 * deadlocking. In case of a multilock, the mutex has to be unlocked
+	 * as many times as it was locked befored it really becomes unlocked.
+	 *
 	 * @param mutex	the mutex to lock.
 	 */
 	virtual void lockMutex(MutexRef mutex) = 0;
@@ -802,31 +909,12 @@ public:
 
 	/** @name Sound */
 	//@{
-	typedef void (*SoundProc)(void *param, byte *buf, int len);
 
 	/**
-	 * Set the audio callback which is invoked whenever samples need to be generated.
-	 * Currently, only the 16-bit signed mode is ever used for Simon & Scumm
-	 * @param proc		pointer to the callback.
-	 * @param param		an arbitrary parameter which is stored and passed to proc.
+	 * Return the audio mixer. For more information, refer to the
+	 * Audio::Mixer documentation.
 	 */
-	virtual bool setSoundCallback(SoundProc proc, void *param) = 0;
-
-	/**
-	 * Remove any audio callback previously set via setSoundCallback, thus effectively
-	 * stopping all audio output immediately.
-	 * @see setSoundCallback
-	 */
-	virtual void clearSoundCallback() = 0;
-
-	/**
-	 * Determine the output sample rate. Audio data provided by the sound
-	 * callback will be played using this rate.
-	 * @note Client code other than the sound mixer should _not_ use this
-	 *       method. Instead, call Mixer::getOutputRate()!
-	 * @return the output sample rate
-	 */
-	virtual int getOutputSampleRate() const = 0;
+	virtual Audio::Mixer *getMixer() = 0;
 
 	//@}
 
@@ -835,6 +923,10 @@ public:
 	/**
 	 * @name Audio CD
 	 * The methods in this group deal with Audio CD playback.
+	 * The default implementation simply does nothing.
+	 * This is the lower level implementation as provided by the
+	 * backends. The engines should use the Audio::AudioCDManager
+	 * class instead of using it directly.
 	 */
 	//@{
 
@@ -842,13 +934,13 @@ public:
 	 * Initialise the specified CD drive for audio playback.
 	 * @return true if the CD drive was inited succesfully
 	 */
-	virtual bool openCD(int drive) = 0;
+	virtual bool openCD(int drive);
 
 	/**
 	 * Poll CD status.
 	 * @return true if CD audio is playing
 	 */
-	virtual bool pollCD() = 0;
+	virtual bool pollCD();
 
 	/**
 	 * Start audio CD playback.
@@ -857,17 +949,17 @@ public:
 	 * @param start_frame	the frame at which playback should start (75 frames = 1 second).
 	 * @param duration		the number of frames to play.
 	 */
-	virtual void playCD(int track, int num_loops, int start_frame, int duration) = 0;
+	virtual void playCD(int track, int num_loops, int start_frame, int duration) {}
 
 	/**
 	 * Stop audio CD playback.
 	 */
-	virtual void stopCD() = 0;
+	virtual void stopCD() {}
 
 	/**
 	 * Update cdrom audio status.
 	 */
-	virtual void updateCD() = 0;
+	virtual void updateCD() {}
 
 	//@}
 
@@ -880,8 +972,11 @@ public:
 
 	/**
 	 * Set a window caption or any other comparable status display to the
-	 * given value.
-	 * @param caption	the window caption to use from now on
+	 * given value. The caption must be a pure ISO LATIN 1 string. Passing a
+	 * string with a different encoding may lead to unexpected behavior,
+	 * even crashes.
+	 *
+	 * @param caption	the window caption to use, as an ISO LATIN 1 string
 	 */
 	virtual void setWindowCaption(const char *caption) {}
 
@@ -891,22 +986,62 @@ public:
 	 * rectangle over the regular screen content; or in a message box beneath
 	 * it; etc.).
 	 *
+	 * Currently, only pure ASCII messages can be expected to show correctly.
+	 *
 	 * @note There is a default implementation which uses a TimedMessageDialog
 	 *       to display the message. Hence implementing this is optional.
 	 *
 	 * @param msg	the message to display on screen
 	 */
-	virtual void displayMessageOnOSD(const char *msg);
+	virtual void displayMessageOnOSD(const char *msg) = 0;
 
-	/** Savefile management. */
-	virtual Common::SaveFileManager *getSavefileManager();
+	/**
+	 * Return the SaveFileManager, used to store and load savestates
+	 * and other modifiable persistent game data. For more information,
+	 * refer to the SaveFileManager documentation.
+	 */
+	virtual Common::SaveFileManager *getSavefileManager() = 0;
+
+	/**
+	 * Returns the FilesystemFactory object, depending on the current architecture.
+	 *
+	 * @return the FSNode factory for the current architecture
+	 */
+	virtual FilesystemFactory *getFilesystemFactory() = 0;
+
+	/**
+	 * Add system specific Common::Archive objects to the given SearchSet.
+	 * E.g. on Unix the dir corresponding to DATA_PATH (if set), or on
+	 * Mac OS X the 'Resource' dir in the app bundle.
+	 *
+	 * @todo Come up with a better name. This one sucks.
+	 *
+	 * @param s		the SearchSet to which the system specific dirs, if any, are added
+	 * @param priority	the priority with which those dirs are added
+	 */
+	virtual void addSysArchivesToSearchSet(Common::SearchSet &s, int priority = 0) {}
+
+	/**
+	 * Open the default config file for reading, by returning a suitable
+	 * ReadStream instance. It is the callers responsiblity to delete
+	 * the stream after use.
+	 */
+	virtual Common::SeekableReadStream *createConfigReadStream() = 0;
+
+	/**
+	 * Open the default config file for writing, by returning a suitable
+	 * WriteStream instance. It is the callers responsiblity to delete
+	 * the stream after use.
+	 *
+	 * May return 0 to indicate that writing to config file is not possible.
+	 */
+	virtual Common::WriteStream *createConfigWriteStream() = 0;
 
 	//@}
 };
 
 
-/** The global OSystem instance. Inited in main(). */
-#define g_system	(&OSystem::instance())
-
+/** The global OSystem instance. Initialised in main(). */
+extern OSystem *g_system;
 
 #endif
