@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/gui/ScrollBarWidget.cpp,v 1.25.2.2 2006/01/18 18:07:28 sev Exp $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/ScrollBarWidget.cpp $
+ * $Id: ScrollBarWidget.cpp 42881 2009-07-29 08:55:17Z fingolfin $
  */
 
-#include "common/stdafx.h"
 #include "ScrollBarWidget.h"
 #include "gui/dialog.h"
-#include "gui/newgui.h"
+#include "gui/GuiManager.h"
 
 namespace GUI {
 
@@ -29,7 +32,7 @@ namespace GUI {
 
 ScrollBarWidget::ScrollBarWidget(GuiObject *boss, int x, int y, int w, int h)
 	: Widget (boss, x, y, w, h), CommandSender(boss) {
-	_flags = WIDGET_ENABLED | WIDGET_TRACK_MOUSE | WIDGET_CLEARBG | WIDGET_WANT_TICKLE;
+	setFlags(WIDGET_ENABLED | WIDGET_TRACK_MOUSE | WIDGET_CLEARBG | WIDGET_WANT_TICKLE);
 	_type = kScrollBarWidget;
 
 	_part = kNoPart;
@@ -60,9 +63,9 @@ void ScrollBarWidget::handleMouseDown(int x, int y, int button, int clickCount)
 		_currentPos++;
 		_draggingPart = kDownArrowPart;
 	} else if (y < _sliderPos) {
-		_currentPos -= _entriesPerPage;
+		_currentPos -= _entriesPerPage - 1;
 	} else if (y >= _sliderPos + _sliderHeight) {
-		_currentPos += _entriesPerPage;
+		_currentPos += _entriesPerPage - 1;
 	} else {
 		_draggingPart = kSliderPart;
 		_sliderDeltaMouseDownPos = y - _sliderPos;
@@ -172,80 +175,30 @@ void ScrollBarWidget::recalc() {
 			UP_DOWN_BOX_HEIGHT + (_h - 2 * UP_DOWN_BOX_HEIGHT - _sliderHeight) * _currentPos / (_numEntries - _entriesPerPage);
 		if (_sliderPos < 0)
 			_sliderPos = 0;
+		setVisible(true);
 	} else {
 		_sliderHeight = _h - 2 * UP_DOWN_BOX_HEIGHT;
 		_sliderPos = UP_DOWN_BOX_HEIGHT;
+		setVisible(false);
 	}
 }
 
-void ScrollBarWidget::drawWidget(bool hilite) {
-	NewGui *gui = &g_gui;
-	int bottomY = _y + _h - UP_DOWN_BOX_HEIGHT;
-	bool isSinglePage = (_numEntries <= _entriesPerPage);
-	OverlayColor color;
-	Graphics::Surface &surf = g_gui.getScreen();
-	const int B = 3;
-	Common::Point p0, p1, p2;
-
-	gui->frameRect(_x, _y, _w, _h, gui->_shadowcolor);
-
+void ScrollBarWidget::drawWidget() {
 	if (_draggingPart != kNoPart)
 		_part = _draggingPart;
 
-	const int arrowSize = (_w / 2 - B + 1);
-
-	//
-	// Up arrow
-	//
-	color = isSinglePage ? gui->_color :
-					(hilite && _part == kUpArrowPart) ? gui->_textcolorhi : gui->_textcolor;
-	gui->frameRect(_x, _y, _w, UP_DOWN_BOX_HEIGHT, gui->_color);
-	p0 = Common::Point(_x + _w / 2, _y + (UP_DOWN_BOX_HEIGHT - arrowSize - 1) / 2);
-	p1 = Common::Point(p0.x - arrowSize, p0.y + arrowSize);
-	p2 = Common::Point(p0.x + arrowSize, p0.y + arrowSize);
-#if 0
-	surf.drawLine(p0.x, p0.y, p1.x, p1.y, color);
-	surf.drawLine(p0.x, p0.y, p2.x, p2.y, color);
-//	surf.drawLine(p1.x, p1.y, p2.x, p2.y, color);
-#else
-	// Evil HACK to draw filled triangle
-	for (; p1.x <= p2.x; ++p1.x)
-		surf.drawLine(p0.x, p0.y, p1.x, p1.y, color);
-#endif
-
-	//
-	// Down arrow
-	//
-	color = isSinglePage ? gui->_color :
-					(hilite && _part == kDownArrowPart) ? gui->_textcolorhi : gui->_textcolor;
-	gui->frameRect(_x, bottomY, _w, UP_DOWN_BOX_HEIGHT, gui->_color);
-	p0 = Common::Point(_x + _w / 2, bottomY + (UP_DOWN_BOX_HEIGHT + arrowSize + 1) / 2);
-	p1 = Common::Point(p0.x - arrowSize, p0.y - arrowSize);
-	p2 = Common::Point(p0.x + arrowSize, p0.y - arrowSize);
-
-#if 0
-	surf.drawLine(p0.x, p0.y, p1.x, p1.y, color);
-	surf.drawLine(p0.x, p0.y, p2.x, p2.y, color);
-//	surf.drawLine(p1.x, p1.y, p2.x, p2.y, color);
-#else
-	// Evil HACK to draw filled triangle
-	for (; p1.x <= p2.x; ++p1.x)
-		surf.drawLine(p0.x, p0.y, p1.x, p1.y, color);
-#endif
-
-	//
-	// Slider
-	//
-	if (!isSinglePage) {
-		gui->fillRect(_x, _y + _sliderPos, _w, _sliderHeight,
-					(hilite && _part == kSliderPart) ? gui->_textcolorhi : gui->_textcolor);
-		gui->frameRect(_x, _y + _sliderPos, _w, _sliderHeight, gui->_color);
-		int y = _y + _sliderPos + _sliderHeight / 2;
-		color = (hilite && _part == kSliderPart) ? gui->_color : gui->_shadowcolor;
-		gui->hLine(_x + 2, y - 2, _x + _w - 3, color);
-		gui->hLine(_x + 2, y, _x + _w - 3, color);
-		gui->hLine(_x + 2, y + 2, _x + _w-3, color);
+	ThemeEngine::ScrollbarState state = ThemeEngine::kScrollbarStateNo;
+	if (_numEntries <= _entriesPerPage) {
+		state = ThemeEngine::kScrollbarStateSinglePage;
+	} else if (_part == kUpArrowPart) {
+		state = ThemeEngine::kScrollbarStateUp;
+	} else if (_part == kDownArrowPart) {
+		state = ThemeEngine::kScrollbarStateDown;
+	} else if (_part == kSliderPart) {
+		state = ThemeEngine::kScrollbarStateSlider;
 	}
+
+	g_gui.theme()->drawScrollbar(Common::Rect(_x, _y, _x+_w, _y+_h), _sliderPos, _sliderHeight, state, _state);
 }
 
 } // End of namespace GUI
