@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +18,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/scummvm/gui/console.cpp,v 1.68.2.2 2006/01/18 18:07:28 sev Exp $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/console.cpp $
+ * $Id: console.cpp 48101 2010-02-21 04:04:13Z bluddy $
  */
 
-#include "common/stdafx.h"
 #include "gui/console.h"
 #include "gui/ScrollBarWidget.h"
+#include "gui/ThemeEval.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "base/version.h"
 
+#include "common/events.h"
 #include "common/system.h"
 
-#include "graphics/font.h"
+#include "graphics/fontman.h"
 
 namespace GUI {
 
@@ -52,30 +57,14 @@ ConsoleDialog::ConsoleDialog(float widthPercent, float heightPercent)
 	: Dialog(0, 0, 1, 1),
 	_widthPercent(widthPercent), _heightPercent(heightPercent) {
 
-	const int screenW = g_system->getOverlayWidth();
-	const int screenH = g_system->getOverlayHeight();
-	
-	_font = FontMan.getFontByUsage(Graphics::FontManager::kConsoleFont);
+	// Reset the line buffer
+	memset(_buffer, ' ', kBufferSize);
 
-	// Calculate the real width/height (rounded to char/line multiples)
-	_w = (uint16)(_widthPercent * screenW);
-	_h = (uint16)((_heightPercent * screenH - 2) / kConsoleLineHeight);
-	_h = _h * kConsoleLineHeight + 2;
-
-	// Add scrollbar
-	int scrollBarWidth;
-	if (screenW >= 400 && screenH >= 300)
-		scrollBarWidth = kBigScrollBarWidth;
-	else
-		scrollBarWidth = kNormalScrollBarWidth;
-	_scrollBar = new ScrollBarWidget(this, _w - scrollBarWidth - 1, 0, scrollBarWidth, _h);
+	// Dummy
+	_scrollBar = new ScrollBarWidget(this, 0, 0, 5, 10);
 	_scrollBar->setTarget(this);
 
-	// Reset the line buffer
-	_lineWidth = (_w - scrollBarWidth - 2) / kConsoleCharWidth;
-	_linesPerPage = (_h - 2) / kConsoleLineHeight;
-	memset(_buffer, ' ', kBufferSize);
-	_linesInBuffer = kBufferSize / _lineWidth;
+	init();
 
 	_currentPos = 0;
 	_scrollLine = _linesPerPage - 1;
@@ -87,6 +76,7 @@ ConsoleDialog::ConsoleDialog(float widthPercent, float heightPercent)
 	_slideMode = kNoSlideMode;
 	_slideTime = 0;
 
+	_promptStartPos = _promptEndPos = -1;
 
 	// Init callback
 	_callbackProc = 0;
@@ -99,13 +89,40 @@ ConsoleDialog::ConsoleDialog(float widthPercent, float heightPercent)
 	for (int i = 0; i < kHistorySize; i++)
 		_history[i][0] = '\0';
 
-	_promptStartPos = _promptEndPos = -1;
-
 	// Display greetings & prompt
 	print(gScummVMFullVersion);
 	print("\nConsole is ready\n");
 }
 
+void ConsoleDialog::init() {
+	const int screenW = g_system->getOverlayWidth();
+	const int screenH = g_system->getOverlayHeight();
+
+	_font = FontMan.getFontByUsage((Graphics::FontManager::FontUsage)
+		g_gui.xmlEval()->getVar("Console.Font", Graphics::FontManager::kConsoleFont));
+
+	_leftPadding = g_gui.xmlEval()->getVar("Globals.Console.Padding.Left", 0);
+	_rightPadding = g_gui.xmlEval()->getVar("Globals.Console.Padding.Right", 0);
+	_topPadding = g_gui.xmlEval()->getVar("Globals.Console.Padding.Top", 0);
+	_bottomPadding = g_gui.xmlEval()->getVar("Globals.Console.Padding.Bottom", 0);
+
+	// Calculate the real width/height (rounded to char/line multiples)
+	_w = (uint16)(_widthPercent * screenW);
+	_h = (uint16)((_heightPercent * screenH - 2) / kConsoleLineHeight);
+
+	_w = _w - _w / 20;
+	_h = _h * kConsoleLineHeight + 2;
+	_x = _w / 40;
+
+	// Set scrollbar dimensions
+	int scrollBarWidth = g_gui.xmlEval()->getVar("Globals.Scrollbar.Width", 0);
+	_scrollBar->resize(_w - scrollBarWidth - 1, 0, scrollBarWidth, _h);
+
+	_pageWidth = (_w - scrollBarWidth - 2 - _leftPadding - _topPadding - scrollBarWidth) / kConsoleCharWidth;
+	_linesPerPage = (_h - 2 - _topPadding - _bottomPadding) / kConsoleLineHeight;
+	_linesInBuffer = kBufferSize / kCharsPerLine;
+}
+
 void ConsoleDialog::slideUpAndClose() {
 	if (_slideMode == kNoSlideMode) {
 		_slideTime = g_system->getMillis();
@@ -114,16 +131,27 @@ void ConsoleDialog::slideUpAndClose() {
 }
 
 void ConsoleDialog::open() {
-	// This dialog will be redrawn a lot, so we store a copy of the blended
-	// background in a separate "canvas", just like in the About dialog.
-	_canvas.pixels = NULL;
-
+	// TODO: find a new way to do this
 	// Initiate sliding the console down. We do a very simple trick to achieve
 	// this effect: we simply move the console dialog just above (outside) the
 	// visible screen area, then shift it down in handleTickle() over a
 	// certain period of time.
 
+	const int screenW = g_system->getOverlayWidth();
+	const int screenH = g_system->getOverlayHeight();
+
+	// Calculate the real width/height (rounded to char/line multiples)
+	uint16 w = (uint16)(_widthPercent * screenW);
+	uint16 h = (uint16)((_heightPercent * screenH - 2) / kConsoleLineHeight);
+
+	h = h * kConsoleLineHeight + 2;
+	w = w - w / 20;
+
+	if (_w != w || _h != h)
+		init();
+
 	_y = -_h;
+
 	_slideTime = g_system->getMillis();
 	_slideMode = kDownSlideMode;
 
@@ -135,68 +163,63 @@ void ConsoleDialog::open() {
 }
 
 void ConsoleDialog::close() {
-	free(_canvas.pixels);
 	Dialog::close();
 }
 
 void ConsoleDialog::drawDialog() {
-	if (!_canvas.pixels) {
-		// Blend over the background. Don't count the time used for
-		// this when timing the slide action.
+	g_gui.theme()->drawDialogBackground(Common::Rect(_x, _y, _x + _w, _y + _h), ThemeEngine::kDialogBackgroundPlain/*_backgroundType*/);
+	// FIXME: for the old theme the frame around the console vanishes
+	// when any action is processed if we enable this
+	// _drawingHints &= ~THEME_HINT_FIRST_DRAW;
 
-		uint32 now = g_system->getMillis();
-		uint32 delta;
-
-		g_gui.blendRect(0, 0, _w, _h, g_gui._bgcolor, 2);
-		g_gui.copyToSurface(&_canvas, 0, 0, _w, _h);
-
-		delta = g_system->getMillis() - now;
-
-		if (_slideTime)
-			_slideTime += delta;
-	}
-
-	g_gui.drawSurface(_canvas, 0, 0);
-
-	// Draw a border
-	g_gui.hLine(_x, _y + _h - 1, _x + _w - 1, g_gui._color);
-
-	// Draw text
-	int start = _scrollLine - _linesPerPage + 1;
-	int y = _y + 2;
-
-	for (int line = 0; line < _linesPerPage; line++) {
-		int x = _x + 1;
-		for (int column = 0; column < _lineWidth; column++) {
-#if 0
-			int l = (start + line) % _linesInBuffer;
-			byte c = buffer(l * _lineWidth + column);
-#else
-			byte c = buffer((start + line) * _lineWidth + column);
-#endif
-			g_gui.drawChar(c, x, y, g_gui._textcolor, _font);
-			x += kConsoleCharWidth;
-		}
-		y += kConsoleLineHeight;
-	}
+	for (int line = 0; line < _linesPerPage; line++)
+		drawLine(line, false);
 
 	// Draw the scrollbar
 	_scrollBar->draw();
-
-	// Finally blit it all to the screen
-	g_gui.addDirtyRect(_x, _y, _w, _h);
 }
 
-void ConsoleDialog::handleScreenChanged() {
-	free(_canvas.pixels);
-	_canvas.pixels = NULL;
+void ConsoleDialog::drawLine(int line, bool restoreBg) {
+	int x = _x + 1 + _leftPadding;
+	int start = _scrollLine - _linesPerPage + 1;
+	int y = _y + 2 + _topPadding;
+	int limit = MIN(_pageWidth, (int)kCharsPerLine);
+
+	y += line * kConsoleLineHeight;
+
+	if (restoreBg) {
+		Common::Rect r(_x, y - 2, _x + _pageWidth * kConsoleCharWidth, y+kConsoleLineHeight);
+		g_gui.theme()->restoreBackground(r);
+		g_gui.theme()->addDirtyRect(r);
+	}
+
+	for (int column = 0; column < limit; column++) {
+#if 0
+		int l = (start + line) % _linesInBuffer;
+		byte c = buffer(l * kCharsPerLine + column);
+#else
+		byte c = buffer((start + line) * kCharsPerLine + column);
+#endif
+		g_gui.theme()->drawChar(Common::Rect(x, y, x+kConsoleCharWidth, y+kConsoleLineHeight), c, _font);
+		x += kConsoleCharWidth;
+	}
+
+	g_gui.theme()->updateScreen();
+}
+
+void ConsoleDialog::reflowLayout() {
+	init();
+
+	_scrollLine = _promptEndPos / kCharsPerLine;
+	if (_scrollLine < _linesPerPage - 1)
+		_scrollLine = _linesPerPage - 1;
+	updateScrollBuffer();
+
+	Dialog::reflowLayout();
 	draw();
 }
 
 void ConsoleDialog::handleTickle() {
-	if (!_canvas.pixels)
-		return;
-
 	uint32 time = g_system->getMillis();
 	if (_caretTime < time) {
 		_caretTime = time + kCaretBlinkTime;
@@ -219,7 +242,7 @@ void ConsoleDialog::handleTickle() {
 			draw();
 		} else if (_slideMode == kUpSlideMode && _y <= -_h) {
 			// End the slide
-			_slideMode = kNoSlideMode;
+			//_slideMode = kNoSlideMode;
 			close();
 		} else
 			draw();
@@ -230,15 +253,15 @@ void ConsoleDialog::handleMouseWheel(int x, int y, int direction) {
 	_scrollBar->handleMouseWheel(x, y, direction);
 }
 
-void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
+void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 	int i;
 
 	if (_slideMode != kNoSlideMode)
 		return;
 
-	switch (keycode) {
-	case '\n':	// enter/return
-	case '\r': {
+	switch (state.keycode) {
+	case Common::KEYCODE_RETURN:
+	case Common::KEYCODE_KP_ENTER: {
 		if (_caretVisible)
 			drawCaret(true);
 
@@ -268,7 +291,7 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 				keepRunning = (*_callbackProc)(this, str, _callbackRefCon);
 
 				// Get rid of the string buffer
-			delete [] str;
+			delete[] str;
 		}
 
 		print(PROMPT);
@@ -279,10 +302,12 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 			slideUpAndClose();
 		break;
 		}
-	case 27:	// escape
+
+	case Common::KEYCODE_ESCAPE:
 		slideUpAndClose();
 		break;
-	case 8:		// backspace
+
+	case Common::KEYCODE_BACKSPACE:
 		if (_caretVisible)
 			drawCaret(true);
 
@@ -291,10 +316,10 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 			killChar();
 		}
 		scrollToCurrent();
-		draw();	// FIXME - not nice to redraw the full console just for one char!
+		drawLine(pos2line(_currentPos));
 		break;
-	case 9: // tab
-	{
+
+	case Common::KEYCODE_TAB: {
 		if (_completionCallbackProc) {
 			int len = _currentPos - _promptStartPos;
 			assert(len >= 0);
@@ -305,53 +330,50 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 				str[i] = buffer(_promptStartPos + i);
 			str[len] = '\0';
 
-			char *completion = 0;
+			Common::String completion;
 			if ((*_completionCallbackProc)(this, str, completion, _callbackRefCon)) {
 				if (_caretVisible)
 					drawCaret(true);
-				insertIntoPrompt(completion);
+				insertIntoPrompt(completion.c_str());
 				scrollToCurrent();
-				draw();
-				delete[] completion;
+				drawLine(pos2line(_currentPos));
 			}
 			delete[] str;
 		}
 		break;
-	}
-	case 127:
-		killChar();
-		draw();
+		}
+
+	// Keypad & special keys
+	//   - if num lock is set, we always go to the default case
+	//   - if num lock is not set, we either fall down to the special key case
+	//     or ignore the key press in case of 0 (INSERT) or 5
+
+	case Common::KEYCODE_KP0:
+	case Common::KEYCODE_KP5:
+		if (state.flags & Common::KBD_NUM)
+			defaultKeyDownHandler(state);
 		break;
-	case 256 + 24:	// pageup
-		if (modifiers == OSystem::KBD_SHIFT) {
-			_scrollLine -= _linesPerPage - 1;
-			if (_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
-				_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
-			updateScrollBuffer();
-			draw();
+
+	case Common::KEYCODE_KP_PERIOD:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_DELETE:
+		if (_currentPos < _promptEndPos) {
+			killChar();
+			drawLine(pos2line(_currentPos));
 		}
 		break;
-	case 256 + 25:	// pagedown
-		if (modifiers == OSystem::KBD_SHIFT) {
-			_scrollLine += _linesPerPage - 1;
-			if (_scrollLine > _promptEndPos / _lineWidth)
-				_scrollLine = _promptEndPos / _lineWidth;
-			updateScrollBuffer();
-			draw();
+
+	case Common::KEYCODE_KP1:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
 		}
-		break;
-	case 256 + 22:	// home
-		if (modifiers == OSystem::KBD_SHIFT) {
-			_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
-			updateScrollBuffer();
-		} else {
-			_currentPos = _promptStartPos;
-		}
-		draw();
-		break;
-	case 256 + 23:	// end
-		if (modifiers == OSystem::KBD_SHIFT) {
-			_scrollLine = _promptEndPos / _lineWidth;
+	case Common::KEYCODE_END:
+		if (state.hasFlags(Common::KBD_SHIFT)) {
+			_scrollLine = _promptEndPos / kCharsPerLine;
 			if (_scrollLine < _linesPerPage - 1)
 				_scrollLine = _linesPerPage - 1;
 			updateScrollBuffer();
@@ -360,39 +382,115 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 		}
 		draw();
 		break;
-	case 273:	// cursor up
-		historyScroll(+1);
-		break;
-	case 274:	// cursor down
+
+	case Common::KEYCODE_KP2:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_DOWN:
 		historyScroll(-1);
 		break;
-	case 275:	// cursor right
-		if (_currentPos < _promptEndPos)
-			_currentPos++;
-		draw();
+
+	case Common::KEYCODE_KP3:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_PAGEDOWN:
+		if (state.hasFlags(Common::KBD_SHIFT)) {
+			_scrollLine += _linesPerPage - 1;
+			if (_scrollLine > _promptEndPos / kCharsPerLine) {
+				_scrollLine = _promptEndPos / kCharsPerLine;
+				if (_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
+					_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
+			}
+			updateScrollBuffer();
+			draw();
+		}
 		break;
-	case 276:	// cursor left
+
+	case Common::KEYCODE_KP4:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_LEFT:
 		if (_currentPos > _promptStartPos)
 			_currentPos--;
+		drawLine(pos2line(_currentPos));
+		break;
+
+	case Common::KEYCODE_KP6:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_RIGHT:
+		if (_currentPos < _promptEndPos)
+			_currentPos++;
+		drawLine(pos2line(_currentPos));
+		break;
+
+	case Common::KEYCODE_KP7:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_HOME:
+		if (state.hasFlags(Common::KBD_SHIFT)) {
+			_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
+			updateScrollBuffer();
+		} else {
+			_currentPos = _promptStartPos;
+		}
 		draw();
 		break;
-	default:
-		if (ascii == '~' || ascii == '#') {
-			slideUpAndClose();
-		} else if (modifiers == OSystem::KBD_CTRL) {
-			specialKeys(keycode);
-		} else if (isprint((char)ascii)) {
-			for (i = _promptEndPos - 1; i >= _currentPos; i--)
-				buffer(i + 1) = buffer(i);
-			_promptEndPos++;
-			putchar((char)ascii);
-			scrollToCurrent();
+
+	case Common::KEYCODE_KP8:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
 		}
+	case Common::KEYCODE_UP:
+		historyScroll(+1);
+		break;
+
+	case Common::KEYCODE_KP9:
+		if (state.flags & Common::KBD_NUM) {
+			defaultKeyDownHandler(state);
+			break;
+		}
+	case Common::KEYCODE_PAGEUP:
+		if (state.hasFlags(Common::KBD_SHIFT)) {
+			_scrollLine -= _linesPerPage - 1;
+			if (_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
+				_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
+			updateScrollBuffer();
+			draw();
+		}
+		break;
+
+	default:
+		defaultKeyDownHandler(state);
 	}
 }
 
-void ConsoleDialog::insertIntoPrompt(const char* str)
-{
+void ConsoleDialog::defaultKeyDownHandler(Common::KeyState &state) {
+	if (state.ascii == '~' || state.ascii == '#') {
+		slideUpAndClose();
+	} else if (state.hasFlags(Common::KBD_CTRL)) {
+		specialKeys(state.keycode);
+	} else if ((state.ascii >= 32 && state.ascii <= 127) || (state.ascii >= 160 && state.ascii <= 255)) {
+		for (int i = _promptEndPos - 1; i >= _currentPos; i--)
+			buffer(i + 1) = buffer(i);
+		_promptEndPos++;
+		putchar((byte)state.ascii);
+		scrollToCurrent();
+	}
+}
+
+void ConsoleDialog::insertIntoPrompt(const char* str) {
 	unsigned int l = strlen(str);
 	for (int i = _promptEndPos - 1; i >= _currentPos; i--)
 		buffer(i + l) = buffer(i);
@@ -416,25 +514,25 @@ void ConsoleDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data
 
 void ConsoleDialog::specialKeys(int keycode) {
 	switch (keycode) {
-	case 'a':
+	case Common::KEYCODE_a:
 		_currentPos = _promptStartPos;
 		draw();
 		break;
-	case 'd':
+	case Common::KEYCODE_d:
 		if (_currentPos < _promptEndPos) {
 			killChar();
 			draw();
 		}
 		break;
-	case 'e':
+	case Common::KEYCODE_e:
 		_currentPos = _promptEndPos;
 		draw();
 		break;
-	case 'k':
+	case Common::KEYCODE_k:
 		killLine();
 		draw();
 		break;
-	case 'w':
+	case Common::KEYCODE_w:
 		killLastWord();
 		draw();
 		break;
@@ -444,8 +542,10 @@ void ConsoleDialog::specialKeys(int keycode) {
 void ConsoleDialog::killChar() {
 	for (int i = _currentPos; i < _promptEndPos; i++)
 		buffer(i) = buffer(i + 1);
-	buffer(_promptEndPos) = ' ';
-	_promptEndPos--;
+	if (_promptEndPos > _promptStartPos) {
+		buffer(_promptEndPos) = ' ';
+		_promptEndPos--;
+	}
 }
 
 void ConsoleDialog::killLine() {
@@ -469,8 +569,10 @@ void ConsoleDialog::killLastWord() {
 
 	for (int i = _currentPos; i < _promptEndPos; i++)
 		buffer(i) = buffer(i + cnt);
-	buffer(_promptEndPos) = ' ';
-	_promptEndPos -= cnt;
+	if (_promptEndPos > _promptStartPos) {
+		buffer(_promptEndPos) = ' ';
+		_promptEndPos -= cnt;
+	}
 }
 
 void ConsoleDialog::addToHistory(const char *str) {
@@ -526,10 +628,10 @@ void ConsoleDialog::historyScroll(int direction) {
 }
 
 void ConsoleDialog::nextLine() {
-	int line = _currentPos / _lineWidth;
+	int line = _currentPos / kCharsPerLine;
 	if (line == _scrollLine)
 		_scrollLine++;
-	_currentPos = (line + 1) * _lineWidth;
+	_currentPos = (line + 1) * kCharsPerLine;
 
 	updateScrollBuffer();
 }
@@ -539,12 +641,12 @@ void ConsoleDialog::nextLine() {
 // a new line is added
 void ConsoleDialog::updateScrollBuffer() {
 	int lastchar = MAX(_promptEndPos, _currentPos);
-	int line = lastchar / _lineWidth;
+	int line = lastchar / kCharsPerLine;
 	int numlines = (line < _linesInBuffer) ? line + 1 : _linesInBuffer;
 	int firstline = line - numlines + 1;
 	if (firstline > _firstLineInBuffer) {
 		// clear old line from buffer
-		for (int i = lastchar; i < (line+1) * _lineWidth; ++i)
+		for (int i = lastchar; i < (line+1) * kCharsPerLine; ++i)
 			buffer(i) = ' ';
 		_firstLineInBuffer = firstline;
 	}
@@ -587,7 +689,7 @@ void ConsoleDialog::putchar(int c) {
 		drawCaret(true);
 
 	putcharIntern(c);
-	draw();	// FIXME - not nice to redraw the full console just for one char!
+	drawLine(pos2line(_currentPos));
 }
 
 void ConsoleDialog::putcharIntern(int c) {
@@ -596,7 +698,7 @@ void ConsoleDialog::putcharIntern(int c) {
 	else {
 		buffer(_currentPos) = (char)c;
 		_currentPos++;
-		if ((_scrollLine + 1) * _lineWidth == _currentPos) {
+		if ((_scrollLine + 1) * kCharsPerLine == _currentPos) {
 			_scrollLine++;
 			updateScrollBuffer();
 		}
@@ -614,7 +716,8 @@ void ConsoleDialog::print(const char *str) {
 }
 
 void ConsoleDialog::drawCaret(bool erase) {
-	int line = _currentPos / _lineWidth;
+	// TODO: use code from EditableWidget::drawCaret here
+	int line = _currentPos / kCharsPerLine;
 	int displayLine = line - _scrollLine + _linesPerPage - 1;
 
 	// Only draw caret if visible
@@ -623,30 +726,22 @@ void ConsoleDialog::drawCaret(bool erase) {
 		return;
 	}
 
-	int x = _x + 1 + (_currentPos % _lineWidth) * kConsoleCharWidth;
-	int y = _y + displayLine * kConsoleLineHeight;
-
-	char c = buffer(_currentPos);
-	if (erase) {
-		g_gui.fillRect(x, y, kConsoleCharWidth, kConsoleLineHeight, g_gui._bgcolor);
-		g_gui.drawChar(c, x, y + 2, g_gui._textcolor, _font);
-	} else {
-		g_gui.fillRect(x, y, kConsoleCharWidth, kConsoleLineHeight, g_gui._textcolor);
-		g_gui.drawChar(c, x, y + 2, g_gui._bgcolor, _font);
-	}
-	g_gui.addDirtyRect(x, y, kConsoleCharWidth, kConsoleLineHeight);
+	int x = _x + 1 + _leftPadding + (_currentPos % kCharsPerLine) * kConsoleCharWidth;
+	int y = _y + _topPadding + displayLine * kConsoleLineHeight;
 
 	_caretVisible = !erase;
+	g_gui.theme()->drawCaret(Common::Rect(x, y, x+1, y+kConsoleLineHeight), erase);
 }
 
 void ConsoleDialog::scrollToCurrent() {
-	int line = _promptEndPos / _lineWidth;
+	int line = _promptEndPos / kCharsPerLine;
 
 	if (line + _linesPerPage <= _scrollLine) {
 		// TODO - this should only occur for loong edit lines, though
 	} else if (line > _scrollLine) {
 		_scrollLine = line;
 		updateScrollBuffer();
+		draw();
 	}
 }
 
